# Generated by Django 3.2.12 on 2022-08-11 01:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crunch', '0006_filesizeattribute'),
    ]

    operations = [
        migrations.AddField(
            model_name='dataset',
            name='base_file_path',
            field=models.CharField(default='', max_length=4096),
            preserve_default=False,
        ),
    ]
