# Generated by Django 3.2.15 on 2022-09-08 04:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crunch', '0010_auto_20220908_0001'),
    ]

    operations = [
        migrations.AddField(
            model_name='dataset',
            name='locked',
            field=models.BooleanField(default=False, help_text='If the dataset is locked then it will not show up in the loop of available datasets.'),
        ),
    ]
