# Generated by Django 3.2.12 on 2022-03-20 05:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crunch', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='attribute',
            name='dataset',
        ),
        migrations.AddField(
            model_name='attribute',
            name='item',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.CASCADE, related_name='attributes', to='crunch.item'),
            preserve_default=False,
        ),
    ]
