# Generated by Django 3.2.12 on 2022-03-20 23:12

from django.db import migrations
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('crunch', '0004_dateattribute'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='slug',
            field=django_extensions.db.fields.AutoSlugField(blank=True, editable=False, max_length=255, populate_from='name', unique=True),
        ),
    ]
