{% if item.has_descendant_latlongattributes %}
    <h2>Map</h2>
    <div class="embed-responsive embed-responsive-16by9">
        <iframe class="embed-responsive-item" src="{% url 'crunch:item-map' slug=item.slug %}" allowfullscreen></iframe>
    </div>        
{% endif %}
