# Generated by Django 3.2.15 on 2022-08-11 02:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crunch', '0008_auto_20220811_1223'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='dataset',
            name='base_file_path',
            field=models.CharField(blank=True, default='', max_length=4096),
        ),
        migrations.AlterField(
            model_name='item',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='status',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]
